/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================
*/

#include "../include/common.h"
#include "../include/io.h"

guint OzEDIT_GetFileType(gchar *filename) {
    struct stat ffs;
    gchar *ext;
    stat(filename,&ffs);
    /* Estrai ed analizza estensione del file */
    ext = strrchr(filename,'.');
    if(ext != NULL) {
	if(g_ascii_strcasecmp(ext,".c") == 0) {
	    return OBJECT_TYPE_CSOURCE;
	} else if(g_ascii_strcasecmp(ext,".h") == 0) {
	    return OBJECT_TYPE_CHEADER;
	} else if(g_ascii_strcasecmp(ext,".java") == 0) {
	    return OBJECT_TYPE_JAVASOURCE;
	} else if(g_ascii_strcasecmp(ext,".pl") == 0) {
	    return OBJECT_TYPE_PERL;
	} else if((g_ascii_strcasecmp(ext,".htm") == 0)||((g_ascii_strcasecmp(ext,".html") == 0))) {
	    return OBJECT_TYPE_HTML;
	} else if(g_ascii_strcasecmp(ext,".cpp") == 0) {
	    return OBJECT_TYPE_CPPSOURCE;
	} else if(g_ascii_strcasecmp(ext,".sh") == 0) {
	    return OBJECT_TYPE_BASH;
	} else {
	    return OBJECT_TYPE_UNKNOWN;        
        }
    } else {
	return OBJECT_TYPE_UNKNOWN;        
    }
}


void OzEDIT_LoadObject(struct _OzEDIT_Object *new_object,gchar *filename) {
    gchar *buffer,*bbuf;
    gsize bread,bwrite;
    FILE *ff;
    GError *error = NULL;
    struct stat ffs;
    /* Verifica se il file esiste e se e' OK ! */
    if(!g_file_test(filename,G_FILE_TEST_IS_REGULAR)) {
	new_object->status = OBJECT_STATUS_NEW;
	new_object->type = OBJECT_TYPE_UNKNOWN;
	OzEDIT_InfoDialog("File %s is not a regular file or not exists.",filename);
	return;
    }
    /* Adesso APRI e carica nel buffer */
    ff = fopen(filename,"r");
    if(ff == NULL) {
	OzEDIT_ErrorDialog("File %s coudn't be opened !",filename);
	new_object->status = OBJECT_STATUS_NEW;
	new_object->type = OBJECT_TYPE_UNKNOWN;
	return;
    }
    new_object->type = OzEDIT_GetFileType(filename);
    stat(filename,&ffs);
#ifdef DEBUG
    g_print("DEBUG: ====== LOAD FILE INFO ======\n");
    g_print("DEBUG: Name: %s\n",filename);
    g_print("DEBUG: Size (in bytes): %d\n",ffs.st_size);
    g_print("DEBUG: Type: %d (%s)\n",new_object->type);
#endif
    buffer = g_malloc0(ffs.st_size);
    fread(buffer,ffs.st_size,1,ff);
#ifdef DEBUG
    g_print("DEBUG: Locale to UTF8\n");
#endif
    /* Converti in UTF-8 */
    bbuf = g_malloc0(ffs.st_size);
    memcpy(bbuf,buffer,ffs.st_size);
    g_free(buffer);
    buffer = g_malloc0(ffs.st_size*2);
    buffer = g_locale_to_utf8(bbuf,ffs.st_size,&bread,&bwrite,&error);
    g_free(bbuf); 
    if(error) {
        g_free(buffer);
	OzEDIT_ErrorDialog("Cannot convert %s to UFT8 :-( Error %s\n",filename,error->message);
	g_error_free(error);
        new_object->status = OBJECT_STATUS_NEW;
	new_object->type = OBJECT_TYPE_UNKNOWN;
        return;		    	
    }
    /* File loaded, now display it */
    gtk_text_buffer_set_text(new_object->text_buffer,buffer,-1);
    gtk_text_buffer_set_modified(new_object->text_buffer,FALSE);
    /* Setta lo stato dell'oggetto */
    new_object->status = OBJECT_STATUS_OK;
    /* Libera la memoria */
    g_free(buffer);
    /* Chiudi e via ! */
    fclose(ff);
}

void OzEDIT_SaveObject(struct _OzEDIT_Object *new_object,gchar *filename) {
    GtkTextIter i_start,i_end;
    gchar *buffer;
    FILE *ff;
    
    if((new_object == NULL)||(filename == NULL)) return; /* Nel caso di errori...evita un SEGFAULT :-) */
    
    /* Verifica se il file esiste */
    if((g_file_test(filename,G_FILE_TEST_IS_REGULAR))&&(g_ascii_strcasecmp(filename,new_object->filename) != 0)) {
	if(OzEDIT_AreYouSureDialog("File %s exists ! Overwrite ?",filename) != GTK_RESPONSE_YES) {
	    return;
	}	
    }
    /* Crea ITERs */
    gtk_text_buffer_get_start_iter(new_object->text_buffer,&i_start);
    gtk_text_buffer_get_end_iter(new_object->text_buffer,&i_end);    
    /* Preleva il contenuto del BUFFER */        
    buffer = gtk_text_buffer_get_text(new_object->text_buffer,&i_start,&i_end,FALSE);
    /* Adesso Apri il file */
    ff = fopen(filename,"w+");
    if(ff == NULL) {
	OzEDIT_ErrorDialog("File %s coudn't be created !",filename);
	return;
    }
#ifdef DEBUG
    g_print("DEBUG: ====== SAVE FILE INFO ======\n");
    g_print("DEBUG: Name: %s\n",filename);
    g_print("DEBUG: Size (in bytes): %d\n",strlen(buffer));
#endif
    /* Scrive nel file */    
    fwrite(buffer,strlen(buffer),1,ff);
    gtk_text_buffer_set_modified(new_object->text_buffer,FALSE);    
    /* Modifica proprieta' dell'oggetto */
    strncpy(new_object->filename,filename,255);
    strncpy(new_object->name,filename,255);
    new_object->type = OzEDIT_GetFileType(filename);
    new_object->status = OBJECT_STATUS_OK;
    new_object->colorized = FALSE;
    /* Chiudi tutto */
    fclose(ff);
}
