/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================

Undo & Redo

*/

#include "../include/common.h"
#include "../include/undoredo.h" 

struct _OzEDIT_UndoRedo {
    gchar *data;
    guint type;
    gint start,end;
};

void OzEDIT_TrimList(GList *);

extern gboolean undo_action,redo_action;

#define UNDOREDO_TYPE_DELETERANGE 1
#define UNDOREDO_TYPE_INSERTTEXT 2

void OzEDIT_DoUndo(struct _OzEDIT_Object *object) {
#warning "DoUndo - Check for bugs !"
    struct _OzEDIT_UndoRedo *undoitem;
    GtkTextIter start,end;
#ifdef DEBUG
    g_print("DEBUG: DoUndo for %s\n",object->name);
#endif
    if(g_queue_is_empty(object->listundo)) return;
    undo_action = TRUE;
    /* preleva ultimo evento UNDO ... */
    undoitem = g_queue_pop_tail(object->listundo);
    /* */
    gtk_text_buffer_begin_user_action(object->text_buffer);
    /* Genera ITERs */
    gtk_text_buffer_get_iter_at_offset(object->text_buffer,&start,undoitem->start);
    if(undoitem->end > 0) gtk_text_buffer_get_iter_at_offset(object->text_buffer,&end,undoitem->end);
    /* Esegui l'undo */
    switch(undoitem->type) {
	case UNDOREDO_TYPE_DELETERANGE: 
#ifdef DEBUG
	        g_print("DEBUG: DoUndo DELETE RANGE %s (%d-%d)\n",undoitem->data,undoitem->start,undoitem->end);
#endif
		gtk_text_buffer_place_cursor(object->text_buffer,&start);
		gtk_text_buffer_insert(object->text_buffer,&start,undoitem->data,strlen(undoitem->data));
	    break;
	case UNDOREDO_TYPE_INSERTTEXT:
#ifdef DEBUG
	        g_print("DEBUG: DoUndo INSERT TEXT %s (%d-%d)\n",undoitem->data,undoitem->start,undoitem->end);
#endif
		 gtk_text_buffer_delete(object->text_buffer,&start,&end);
	    break;
	default:
	    break;
    }
    /* */
    gtk_text_buffer_end_user_action(object->text_buffer);
    /* Aggiungilo alla lista REDO ! */
    g_queue_push_tail(object->listredo,(struct _OzEDIT_UndoRedo *)undoitem);
    /* Fine */
    undo_action = FALSE;
}

void OzEDIT_DoRedo(struct _OzEDIT_Object *object) {
#warning "DoRedo - Ckeck for bugs !"
    struct _OzEDIT_UndoRedo *redoitem;
    GtkTextIter start,end;
#ifdef DEBUG
    g_print("DEBUG: DoRedo for %s\n",object->name);
#endif
    /* preleva ultimo evento REDO ... */
    if(g_queue_is_empty(object->listredo)) return;
    redo_action = TRUE;
    redoitem = g_queue_pop_tail(object->listredo);
    /* */
    gtk_text_buffer_begin_user_action(object->text_buffer);
    /* Genera ITERs */
    gtk_text_buffer_get_iter_at_offset(object->text_buffer,&start,redoitem->start);
    if(redoitem->end > 0) gtk_text_buffer_get_iter_at_offset(object->text_buffer,&end,redoitem->end-1);
    /* Esegui il redo */
    switch(redoitem->type) {
	case UNDOREDO_TYPE_INSERTTEXT:
#ifdef DEBUG
	        g_print("DEBUG: DoRedo INSERT TEXT %s (%d-%d)\n",redoitem->data,redoitem->start,redoitem->end);
#endif
		gtk_text_buffer_place_cursor(object->text_buffer,&start);
		gtk_text_buffer_insert(object->text_buffer,&start,redoitem->data,strlen(redoitem->data));
	    break;
	case UNDOREDO_TYPE_DELETERANGE: 
#ifdef DEBUG
	        g_print("DEBUG: DoRedo DELETE RANGE %s (%d-%d)\n",redoitem->data,redoitem->start,redoitem->end);
#endif
		 gtk_text_buffer_delete(object->text_buffer,&start,&end);
	    break;
	default:
	    break;
    }
    /* */
    gtk_text_buffer_end_user_action(object->text_buffer);
    /* Aggiungilo alla lista UNDO ! */
    g_queue_push_tail(object->listundo,(struct _OzEDIT_UndoRedo *)redoitem);
    /* Fine */
    redo_action = FALSE;
}


void OzEDIT_CBUndoRedoDeleteRange(GtkTextBuffer *textbuffer,GtkTextIter *start,GtkTextIter *end,struct _OzEDIT_Object *object) {
    struct _OzEDIT_UndoRedo *undoredo;
    if((undo_action)||(redo_action)) return;
#ifdef DEBUG
    g_print("DEBUG: CBUndoRedoDeleteRange for %s -> %s\n",object->name,gtk_text_iter_get_text(start,end));
#endif
    undoredo = g_malloc0(sizeof(struct _OzEDIT_UndoRedo));    
    undoredo->type = UNDOREDO_TYPE_DELETERANGE;
    undoredo->start = gtk_text_iter_get_offset(start);
    undoredo->end = gtk_text_iter_get_offset(end) + 1;
    undoredo->data = g_malloc0(strlen(gtk_text_iter_get_text(start,end)));
    g_strlcpy(undoredo->data,gtk_text_iter_get_text(start,end),undoredo->end - undoredo->start);
    g_queue_push_tail(object->listundo,(struct _OzEDIT_UndoRedo *)undoredo);
}

void OzEDIT_CBUndoRedoInsertText(GtkTextBuffer *textbuffer,GtkTextIter *start,gchar *arg2,gint arg3,struct _OzEDIT_Object *object) {
    struct _OzEDIT_UndoRedo *undoredo;
    if((undo_action)||(redo_action)) return;
#ifdef DEBUG
    g_print("DEBUG: CBUndoRedoInsertText for %s -> %s\n",object->name,arg2);
#endif
    undoredo = g_malloc0(sizeof(struct _OzEDIT_UndoRedo));    
    undoredo->type = UNDOREDO_TYPE_INSERTTEXT;
    undoredo->start = gtk_text_iter_get_offset(start);
    undoredo->end = undoredo->start + arg3;
    undoredo->data = g_malloc0(strlen(arg2));
    g_strlcpy(undoredo->data,arg2,strlen(arg2));
    g_queue_push_tail(object->listundo,(struct _OzEDIT_UndoRedo *)undoredo);
}

/* Inzializza Undo & Redo */
void OzEDIT_UndoRedoInit(struct _OzEDIT_Object *object) {
    object->listundo=g_queue_new();
    object->listredo=g_queue_new();
    undo_action = FALSE;
    redo_action = FALSE;
    g_signal_connect(G_OBJECT(object->text_buffer),"delete-range",G_CALLBACK(OzEDIT_CBUndoRedoDeleteRange),object);
    g_signal_connect(G_OBJECT(object->text_buffer),"insert-text",G_CALLBACK(OzEDIT_CBUndoRedoInsertText),object);
}

